@@ -1,3 +1,4 @@
+# SPDX-License-Identifier: GPL-3.0-only
 from gi.repository import Gtk
 from passwordsafe.unlock_database import UnlockDatabase
 
@@ -37,7 +38,7 @@ class CreatedDatabase:
     # Events
     #
 
-    def on_finish_button_clicked(self, widget):
+    def on_finish_button_clicked(self, _widget):
         self.stack.destroy()
         UnlockDatabase(
             self.window, self.parent_widget,
