@@ -1,30 +1,23 @@
-from gettext import gettext as _
-from gi.repository import Gtk, GLib
-from random import randint
+# SPDX-License-Identifier: GPL-3.0-only
+from __future__ import annotations
+
 import ntpath
-import passwordsafe.config_manager
-import passwordsafe.keyfile_generator
-import passwordsafe.password_generator
+import os
 import threading
 import time
-import os
+from gettext import gettext as _
+
+from gi.repository import Gdk, GLib, Gtk, Handy
+
+import passwordsafe.config_manager as config
+import passwordsafe.keyfile_generator
+import passwordsafe.password_generator
 
 
 class DatabaseSettingsDialog:
-    dialog = NotImplemented
-
-    unlocked_database = NotImplemented
-    database_manager = NotImplemented
-    builder = NotImplemented
-    stack = NotImplemented
-
-    auth_apply_button = NotImplemented
-    encryption_apply_button = NotImplemented
-    generate_keyfile_button = NotImplemented
+    # pylint: disable=too-many-instance-attributes
 
     new_password = NotImplemented
-
-    selected_keyfile_hash = NotImplemented
     new_keyfile_path = NotImplemented
 
     entries_number = NotImplemented
@@ -40,32 +33,31 @@ class DatabaseSettingsDialog:
         self.builder = Gtk.Builder()
         self.builder.add_from_resource("/org/gnome/PasswordSafe/database_settings_dialog.ui")
 
-        self.assemble_dialog()
-        # self.assemble_encryption_page()
-        self.set_detail_values()
+        self.window = self.builder.get_object("database_settings_window")
+        self.auth_apply_button = self.builder.get_object("auth_apply_button")
 
-        stats_thread = threading.Thread(target=self.start_stats_thread)
-        stats_thread.daemon = True
-        stats_thread.start()
+        self.stack = self.builder.get_object("dbsd_stack")
+        self.encryption_apply_button = self.builder.get_object("encryption_apply_button")
+        self.select_keyfile_button = self.builder.get_object("select_keyfile_button")
+        self.generate_keyfile_button = self.builder.get_object("generate_keyfile_button")
+
+        self.__setup_widgets()
+        self.__setup_signals()
+
+        # TODO Implement this feature: Change the encryption algorithm of the Safe
+        # https://gitlab.gnome.org/World/PasswordSafe/-/issues/256
+        # self.assemble_encryption_page()
 
     #
     # Dialog Creation
     #
 
-    def assemble_dialog(self):
-        self.dialog = self.builder.get_object("database_settings_dialog")
-        self.stack = self.builder.get_object("dbsd_stack")
-
-        self.stack.set_visible_child(self.stack.get_child_by_name("auth_page"))
-
-        # Apply Buttons
-        self.auth_apply_button = self.builder.get_object("auth_apply_button")
+    def __setup_signals(self) -> None:
         self.auth_apply_button.connect("clicked", self.on_auth_apply_button_clicked)
-        self.auth_apply_button.set_sensitive(False)
-
-        self.encryption_apply_button = self.builder.get_object("encryption_apply_button")
-        self.encryption_apply_button.connect("clicked", self.on_encryption_apply_button_clicked)
-        self.encryption_apply_button.set_sensitive(False)
+        self.encryption_apply_button.connect(
+            "clicked", self.on_encryption_apply_button_clicked
+        )
+        self.window.connect("key-press-event", self.__on_key_press_event)
 
         # Password Section
         self.builder.get_object("current_password_entry").connect("changed", self.on_password_entry_changed)
@@ -74,16 +66,21 @@ class DatabaseSettingsDialog:
         self.builder.get_object("confirm_password_entry").connect("changed", self.on_password_entry_changed)
         self.builder.get_object("confirm_password_entry").connect("icon-press", self.on_show_password)
 
-        # Keyfile Section
-        select_keyfile_button = self.builder.get_object("select_keyfile_button")
-        select_keyfile_button.connect("clicked", self.on_keyfile_select_button_clicked)
-
-        self.generate_keyfile_button = self.builder.get_object("generate_keyfile_button")
         self.generate_keyfile_button.connect("clicked", self.on_keyfile_generator_button_clicked)
+        self.select_keyfile_button.connect("clicked", self.on_keyfile_select_button_clicked)
+
+        self.database_manager.connect(
+            "notify::locked", self.__on_locked
+        )
+
+    def __setup_widgets(self) -> None:
+        self.stack.set_visible_child(self.stack.get_child_by_name("auth_page"))
+        self.auth_apply_button.set_sensitive(False)
+        self.encryption_apply_button.set_sensitive(False)
 
         if self.database_manager.keyfile_hash is NotImplemented:
             self.generate_keyfile_button.set_sensitive(True)
-            select_keyfile_button.set_sensitive(False)
+            self.select_keyfile_button.set_sensitive(False)
         else:
             self.generate_keyfile_button.set_sensitive(False)
 
@@ -95,60 +92,69 @@ class DatabaseSettingsDialog:
         level_bar.add_offset_value("secure", 5.0)
 
         # Dialog
-        self.dialog.set_modal(True)
-        self.dialog.set_transient_for(self.unlocked_database.window)
-        self.dialog.present()
+        self.window.set_modal(True)
+        self.window.set_transient_for(self.unlocked_database.window)
 
-        self.unlocked_database.database_settings_dialog = self.dialog
-        self.dialog.connect("delete-event", self.on_dialog_quit)
+        self.set_detail_values()
+
+        stats_thread = threading.Thread(target=self.start_stats_thread)
+        stats_thread.daemon = True
+        stats_thread.start()
+
+    def present(self):
+        self.window.present()
 
     #
     # Password Section
     #
 
-    def on_password_entry_changed(self, entry):
+    def on_password_entry_changed(self, entry: Gtk.Entry) -> None:
+        """CB if password entry (existing or new) has changed"""
+
         self.unlocked_database.start_database_lock_timer()
 
         current_password = self.builder.get_object("current_password_entry").get_text()
-        new_password = self.builder.get_object("new_password_entry").get_text()
-        confirm_password = self.builder.get_object("confirm_password_entry").get_text()
+        new_password_entry = self.builder.get_object("new_password_entry")
+        new_password = new_password_entry.get_text()
+        conf_password_entry = self.builder.get_object("confirm_password_entry")
+        conf_password = conf_password_entry.get_text()
 
         self.new_password = NotImplemented
 
-        if entry.get_name() == "new_entry":
+        # Update the quality level bar
+        if entry.get_name() == "new_entry" and new_password:
             level = passwordsafe.password_generator.strength(new_password)
-            self.builder.get_object("password_level_bar").set_value(float(level))
+            self.builder.get_object("password_level_bar").set_value(level or 0)
 
-        if self.database_manager.password == "" or self.database_manager.password is None:
-            if new_password == "" or confirm_password == "":
-                self.auth_apply_button.set_sensitive(False)
-                return
-        else:
-            if current_password == "" or new_password == "" or confirm_password == "":
-                self.auth_apply_button.set_sensitive(False)
-                return
-
-        if new_password != confirm_password:
+        if new_password != conf_password:
             self.auth_apply_button.set_sensitive(False)
-            self.builder.get_object("new_password_entry").get_style_context().add_class("error")
-            self.builder.get_object("confirm_password_entry").get_style_context().add_class("error")
+            new_password_entry.get_style_context().add_class("error")
+            conf_password_entry.get_style_context().add_class("error")
             return
+        new_password_entry.get_style_context().remove_class("error")
+        conf_password_entry.get_style_context().remove_class("error")
 
-        self.builder.get_object("new_password_entry").get_style_context().remove_class("error")
-        self.builder.get_object("confirm_password_entry").get_style_context().remove_class("error")
-
-        if current_password != self.database_manager.password and self.database_manager.password != "" and self.database_manager.password is not None:
+        if (self.database_manager.password != current_password
+           or (not new_password or not conf_password)):
+            # 1) existing db password != current one entered
+            # 2) Nothing entered for new and/or confirmed password
             self.auth_apply_button.set_sensitive(False)
             return
 
         self.auth_apply_button.set_sensitive(True)
-        self.new_password = confirm_password
+        self.new_password = new_password
 
-    def on_generate_password(self, widget, position, eventbutton):
+    def on_generate_password(self, _widget, _position, _eventbutton):
         new_password_entry = self.builder.get_object("new_password_entry")
         confirm_password_entry = self.builder.get_object("confirm_password_entry")
 
-        generated_password = passwordsafe.password_generator.generate(randint(18, 24), True, True, True, True)
+        use_lowercase = config.get_generator_use_lowercase()
+        use_uppercase = config.get_generator_use_uppercase()
+        use_numbers = config.get_generator_use_numbers()
+        use_symbols = config.get_generator_use_symbols()
+        length = config.get_generator_length()
+
+        generated_password = passwordsafe.password_generator.generate(length, use_uppercase, use_lowercase, use_numbers, use_symbols)
 
         new_password_entry.set_text(generated_password)
         confirm_password_entry.set_text(generated_password)
@@ -156,7 +162,7 @@ class DatabaseSettingsDialog:
         new_password_entry.set_visibility(True)
         confirm_password_entry.set_visibility(True)
 
-    def on_show_password(self, widget, position, eventbutton):
+    def on_show_password(self, _widget, _position, _eventbutton):
         new_password_entry = self.builder.get_object("new_password_entry")
         confirm_password_entry = self.builder.get_object("confirm_password_entry")
 
@@ -171,11 +177,11 @@ class DatabaseSettingsDialog:
     # Keyfile Section
     #
 
-    def on_keyfile_select_button_clicked(self, button):
+    def on_keyfile_select_button_clicked(self, button: Gtk.Button) -> None:
         self.unlocked_database.start_database_lock_timer()
         select_dialog = Gtk.FileChooserNative.new(
             # NOTE: Filechooser title for choosing current used keyfile
-            _("Choose current keyfile"), self.dialog, Gtk.FileChooserAction.OPEN,
+            _("Choose current keyfile"), self.window, Gtk.FileChooserAction.OPEN,
             _("Open"), None)
         select_dialog.set_modal(True)
         select_dialog.set_local_only(False)
@@ -192,12 +198,17 @@ class DatabaseSettingsDialog:
 
         if response == Gtk.ResponseType.ACCEPT:
             selected_keyfile = select_dialog.get_filename()
-            self.keyfile_hash = self.database_manager.create_keyfile_hash(selected_keyfile)
+            keyfile_hash: str = self.database_manager.create_keyfile_hash(
+                selected_keyfile
+            )
 
-            if self.keyfile_hash == self.database_manager.keyfile_hash or self.database_manager.keyfile_hash is NotImplemented:
+            if (
+                keyfile_hash == self.database_manager.keyfile_hash
+                or self.database_manager.keyfile_hash is NotImplemented
+            ):
                 self.generate_keyfile_button.set_sensitive(True)
 
-                if self.keyfile_hash == self.database_manager.keyfile_hash:
+                if keyfile_hash == self.database_manager.keyfile_hash:
                     self.new_keyfile_path = selected_keyfile
 
                 button.set_sensitive(False)
@@ -209,11 +220,11 @@ class DatabaseSettingsDialog:
                 button.add(Gtk.Image.new_from_icon_name("edit-delete-symbolic", Gtk.IconSize.BUTTON))
                 button.show_all()
 
-    def on_keyfile_generator_button_clicked(self, button):
+    def on_keyfile_generator_button_clicked(self, _button: Gtk.Button) -> None:
         self.unlocked_database.start_database_lock_timer()
         save_dialog = Gtk.FileChooserNative.new(
             # NOTE: Filechooser title for generating a new keyfile
-            _("Choose location for keyfile"), self.dialog, Gtk.FileChooserAction.SAVE,
+            _("Choose location for keyfile"), self.window, Gtk.FileChooserAction.SAVE,
             _("Generate"), None)
         save_dialog.set_do_overwrite_confirmation(True)
         save_dialog.set_current_name(_("Keyfile"))
@@ -240,7 +251,6 @@ class DatabaseSettingsDialog:
             self.generate_keyfile_button.show_all()
 
             self.new_keyfile_path = save_dialog.get_filename()
-            generator_thread = NotImplemented
 
             if self.new_password is NotImplemented:
                 generator_thread = threading.Thread(target=passwordsafe.keyfile_generator.generate_keyfile, args=(self.new_keyfile_path, False, self, False))
@@ -268,13 +278,11 @@ class DatabaseSettingsDialog:
                 self.database_manager.set_database_keyfile(None)
                 self.database_manager.keyfile_hash = NotImplemented
 
-            self.database_manager.set_database_password(self.new_password)
             self.database_manager.password = self.new_password
 
         if self.new_keyfile_path is not NotImplemented:
             if self.new_password is NotImplemented:
-                self.database_manager.set_database_password(None)
-                self.database_manager.password = ""
+                self.database_manager.password = None
 
             self.database_manager.set_database_keyfile(str(self.new_keyfile_path))
             self.database_manager.keyfile_hash = self.database_manager.create_keyfile_hash(str(self.new_keyfile_path))
@@ -329,13 +337,12 @@ class DatabaseSettingsDialog:
         self.generate_keyfile_button.show_all()
 
         self.new_password = NotImplemented
-        self.selected_keyfile_hash = NotImplemented
         self.new_keyfile_path = NotImplemented
 
         self.auth_apply_button.set_label(_("Apply Changes"))
         self.auth_apply_button.set_sensitive(False)
 
-    def on_encryption_apply_button_clicked(self, button):
+    def on_encryption_apply_button_clicked(self, _button):
         if self.new_encryption_algorithm is not NotImplemented:
             self.database_manager.db.encryption_algorithm = self.new_encryption_algorithm
 
@@ -449,7 +456,7 @@ class DatabaseSettingsDialog:
         enc_alg_list.connect("row-activated", self.on_encryption_changed)
         der_alg_list.connect("row-activated", self.on_derivation_changed)
 
-    def on_encryption_changed(self, list_box, row):
+    def on_encryption_changed(self, _list_box, row):
         self.new_encryption_algorithm = NotImplemented
 
         if row.get_name() != "chacha20" and row.get_name() != "twofish" and row.get_name() != "aes256":
@@ -467,7 +474,7 @@ class DatabaseSettingsDialog:
         else:
             self.encryption_apply_button.set_sensitive(True)
 
-    def on_derivation_changed(self, list_box, row):
+    def on_derivation_changed(self, _list_box, row):
         self.new_derivation_algorithm = NotImplemented
 
         if row.get_name() != "argon2" and row.get_name() != "aeskdf":
@@ -485,15 +492,24 @@ class DatabaseSettingsDialog:
 
         self.new_derivation_algorithm = version_tuple
 
-        if self.database_manager.db.encryption_algorithm == self.builder.get_object("enc_alg_list_box").get_selected_row().get_name() and self.database_manager.db.version == version_tuple:
+        enc_alg: str = (
+            self.builder.get_object("enc_alg_list_box").get_selected_row().get_name()
+        )
+        if (
+            self.database_manager.db.encryption_algorithm == enc_alg
+            and self.database_manager.db.version == version_tuple
+        ):
             self.encryption_apply_button.set_sensitive(False)
         else:
             self.encryption_apply_button.set_sensitive(True)
 
-    #
-    # Tools
-    #
-
-    def on_dialog_quit(self, window, event):
-        self.unlocked_database.database_settings_dialog = NotImplemented
+    def __on_locked(self, database_manager, _value):
+        locked = database_manager.props.locked
+        if locked:
+            self.window.close()
 
+    def __on_key_press_event(self, _window: Handy.Window, event: Gtk.Event) -> bool:
+        if event.keyval == Gdk.KEY_Escape:
+            self.window.close()
+            return Gdk.EVENT_STOP
+        return Gdk.EVENT_PROPAGATE
