@@ -1,7 +1,12 @@
-from gi.repository import Gtk
+# SPDX-License-Identifier: GPL-3.0-only
+from __future__ import annotations
+
+from gi.repository import Gtk, Gdk, Handy
 
 
 class NotesDialog():
+    # pylint: disable=too-many-instance-attributes
+
     dialog = NotImplemented
 
     unlocked_database = NotImplemented
@@ -29,15 +34,15 @@ class NotesDialog():
         self.dialog.set_transient_for(self.unlocked_database.window)
         self.dialog.present()
 
-        self.unlocked_database.references_dialog = self.dialog
+        self.unlocked_database.notes_dialog = self.dialog
         self.dialog.connect("delete-event", self.on_dialog_quit)
+        self.dialog.connect("key-press-event", self.__on_key_press_event)
 
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+        scrolled_page = self.unlocked_database.get_current_page()
         scrolled_page.notes_dialog_value_entry = self.builder.get_object("value_entry")
         scrolled_page.notes_dialog_value_entry.get_buffer().connect("changed", self.on_value_entry_changed)
 
         self.builder.get_object("copy_button").connect("clicked", self.on_copy_button_clicked)
-        self.builder.get_object("close_button").connect("clicked", self.on_close_button_clicked)
 
         # Search
         self.search_button = self.builder.get_object("search_button")
@@ -63,7 +68,7 @@ class NotesDialog():
         self.search_button.add_accelerator("clicked", self.accelerators, key, mod, Gtk.AccelFlags.VISIBLE)
 
     def update_value_entry(self):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+        scrolled_page = self.unlocked_database.get_current_page()
 
         buffer_text = scrolled_page.notes_property_value_entry.get_buffer().get_text(
             scrolled_page.notes_property_value_entry.get_buffer().get_start_iter(),
@@ -77,26 +82,23 @@ class NotesDialog():
 
     def on_value_entry_changed(self, widget):
         self.unlocked_database.start_database_lock_timer()
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+        scrolled_page = self.unlocked_database.get_current_page()
 
         scrolled_page.notes_property_value_entry.get_buffer().set_text(
             widget.get_text(widget.get_start_iter(), widget.get_end_iter(), False)
         )
 
-    def on_copy_button_clicked(self, button):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+    def on_copy_button_clicked(self, _button):
+        scrolled_page = self.unlocked_database.get_current_page()
 
         self.unlocked_database.send_to_clipboard(
             scrolled_page.notes_dialog_value_entry.get_buffer().get_text(
                 scrolled_page.notes_dialog_value_entry.get_buffer().get_start_iter(),
                 scrolled_page.notes_dialog_value_entry.get_buffer().get_end_iter(),
                 False)
-            )
+        )
 
-    def on_close_button_clicked(self, button):
-        self.dialog.destroy()
-
-    def on_search_button_toggled(self, button):
+    def on_search_button_toggled(self, _button):
         if self.search_stopped is True:
             return
         self.toggle_search_bar()
@@ -108,7 +110,7 @@ class NotesDialog():
             self.search_bar.set_search_mode(True)
 
     def on_search_entry_changed(self, entry):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+        scrolled_page = self.unlocked_database.get_current_page()
         notes_buffer = scrolled_page.notes_dialog_value_entry.get_buffer()
 
         scrolled_page.notes_dialog_value_entry.get_buffer().remove_all_tags(
@@ -117,10 +119,10 @@ class NotesDialog():
         )
 
         start = notes_buffer.get_start_iter()
-        if entry.get_text() is not "":
+        if entry.get_text():
             self.do_search(notes_buffer, entry.get_text(), start)
 
-    def on_search_stopped(self, entry):
+    def on_search_stopped(self, _entry):
         self.search_stopped = True
         self.search_button.set_active(False)
         self.search_stopped = False
@@ -138,8 +140,14 @@ class NotesDialog():
             notes_buffer.apply_tag(self.tag, match_start, match_end)
             self.do_search(notes_buffer, keyword, match_end)
 
-    def on_dialog_quit(self, window, event):
+    def on_dialog_quit(self, _window, _event):
         self.unlocked_database.notes_dialog = NotImplemented
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
+        scrolled_page = self.unlocked_database.get_current_page()
         scrolled_page.notes_dialog_value_entry = NotImplemented
 
+    def __on_key_press_event(self, _window: Handy.Window, event: Gtk.Event) -> bool:
+        if event.keyval == Gdk.KEY_Escape:
+            if not self.search_bar.get_search_mode():
+                self.dialog.close()
+                return Gdk.EVENT_STOP
+        return Gdk.EVENT_PROPAGATE
